class SettingApiUrl:
    SETTINGS = "/api/v3/settings"
    SETTINGS_PAYMENT_PROCESSOR = "/api/v2/settings/payment_processors"
    SETTINGS_PRICING_LEVELS = "/api/v3/settings/pricing_levels"
    SETTINGS_ITEM_GROUPS = "/api/v3/settings/item_groups"
    SETTINGS_CURRENCIES = "/api/v3/settings/currencies"
    SETTINGS_TAXES = "/api/v2/settings/taxes"
    SETTINGS_WAREHOUSES = "/api/v2/settings/warehouses"
    SETTINGS_COMPONENT = "/api/v2/settings/components"
    SETTINGS_SHIPPING_PROFILES = "/api/v2/settings/shipping-profiles"
    SETTINGS_VARIATIONS = "/api/v2/settings/variations"
    SETTINGS_VARIATIONS_CREATE = "/api/v2/settings/variations"
    SETTINGS_VARIATIONS_UPDATE = "/api/v2/settings/variations/{uuid}"
    SETTINGS_DISCOUNT_PROFILES = "/api/v2/settings/discount_profiles"


import os.path
import pathlib
from os.path import exists
from setuptools import setup, find_packages

CURRENT_DIR = pathlib.Path(__file__).parent
long_description = ""
readme_md_file = os.path.join(CURRENT_DIR, "README.md")
if exists(readme_md_file):
    long_description = pathlib.Path(readme_md_file).read_text()


def get_dependencies():
    return ["requests", "setuptools", "peewee", "mysql-connector-python"]


setup(
    name='exsited',
    version='1.0.6',
    description='Exsited SDK',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Ashiq Rahman',
    author_email='ashiq@webalive.com.au',
    url='https://github.com/exsited/exsited-python',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=get_dependencies(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    project_urls={
        'Documentation': 'https://stageexsited.mywebcommander.com/exsited-sdk-introduction',
    },
)


from enum import Enum


class RequestType(Enum):
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"


class HTTPConst:
    APPLICATION_JSON = "application/json"
    SUCCESS = "success"
    ERROR = "error"
    UTF8_ENCODING = "utf-8"



import os

from exsited.exsited.account.account_api_url import AccountApiUrl
from exsited.exsited.account.dto.account_dto import AccountCreateDTO, AccountDetailsDTO, AccountListDTO, \
    AccountUpdateInformationDTO, AccountContactsDTO, PaymentMethodsAddDTO, PaymentMethodsDetailsDTO, \
    PaymentCardMethodsAddDTO, \
    PaymentMethodsListDTO, AccountCancelDTO, AccountCancelDataDTO, AccountReactivateDataDTO, AccountReactivateDTO, \
    AccountContactsTypeDTO, AccountContactUpdateDTO, AccountContactsUpdateDTO, AccountReactiveResponseDTO, \
    AccountAddressesAddDTO, AccountCancelResponseDTO, AccountAddressDetailsDTO, AccountAddressUuidDetailsDTO, \
    AccountNoteDetailsDTO, AccountNoteUuidDetailsDTO, AccountNoteUuidFileDetailsDTO, AccountNoteUuidFileUuidDetailsDTO, \
    AccountImageDetailsDTO, AccountNoteResponseDetailsDTO, AccountBillingPreferencesRequestDTO, \
    AccountBillingPreferencesResponseDTO, AccountAddressRequestDTO, AccountAddressResponseDTO, EmailSentResponseDTO, \
    EmailGetListResponseDTO, EmailGetDetailsResponseDTO
from exsited.exsited.account.dto.account_nested_dto import AccountContactsUpdate, ContactDTO
from exsited.exsited.common.common_enum import SortDirection
from exsited.common.sdk_util import SDKUtil
from exsited.http.ab_rest_processor import ABRestProcessor
import mimetypes

class Account(ABRestProcessor):

    def create(self, request_data: AccountCreateDTO) -> AccountDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNTS, request_obj=request_data, response_obj=AccountDetailsDTO())
        return response

    def add_image(self, file_url: str, account_id: str) -> AccountImageDetailsDTO:
        mime_type, _ = mimetypes.guess_type(file_url)
        if not mime_type:
            raise ValueError(f"Unable to determine MIME type for file: {file_url}")

        with open(file_url, "rb") as image_file:
            files = {
                "image": (file_url.split("\\")[-1], image_file, mime_type)
            }
            response = self.post(url=AccountApiUrl.ACCOUNTS_IMAGE.format(id=account_id), file=files, response_obj=AccountImageDetailsDTO())
        return response

    def add_notes_uuid(self, file_url: str, account_id: str, note_uuid:str) -> AccountNoteResponseDetailsDTO:
        if file_url:
            mime_type, _ = mimetypes.guess_type(file_url)
            if not mime_type:
                raise ValueError(f"Unable to determine MIME type for file: {file_url}")

            with open(file_url, "rb") as image_file:
                files = {
                    "file": (file_url.split("\\")[-1], image_file, mime_type)
                }

                response = self.post(
                    url=AccountApiUrl.ACCOUNTS_NOTE_UUID.format(id=account_id,uuid=note_uuid),
                    file=files,
                    response_obj=AccountNoteResponseDetailsDTO()
                )
            return response


    def add_notes(self, file_url: str = None, account_id: str = "", datas: str = "") -> AccountNoteResponseDetailsDTO:
        note = {
            "note": datas
        }

        if file_url:
            mime_type, _ = mimetypes.guess_type(file_url)
            if not mime_type:
                raise ValueError(f"Unable to determine MIME type for file: {file_url}")

            with open(file_url, "rb") as image_file:
                files = {
                    "file": (file_url.split("\\")[-1], image_file, mime_type)
                }

                response = self.post(
                    url=AccountApiUrl.ACCOUNTS_NOTE.format(id=account_id),
                    data=note,
                    file=files,
                    response_obj=AccountNoteResponseDetailsDTO()
                )
        else:
            response = self.post(
                url=AccountApiUrl.ACCOUNTS_NOTE.format(id=account_id),
                data=note,
                response_obj=AccountNoteResponseDetailsDTO()
            )

        return response

    def create_v3(self, request_data: AccountCreateDTO) -> AccountDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNTS_V3, request_obj=request_data, response_obj=AccountDetailsDTO())
        return response

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> AccountListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=AccountApiUrl.ACCOUNTS, params=params, response_obj=AccountListDTO())
        return response

    def details(self, id: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_V3 + f"/{id}", response_obj=AccountDetailsDTO())
        return response

    def get_image(self, id: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_IMAGE.format(id=id), response_obj=AccountImageDetailsDTO())
        return response

    def address_details(self, id: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_ADDRESS.format(id=id), response_obj=AccountAddressDetailsDTO())
        return response

    def note_details(self, id: str, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> AccountDetailsDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=AccountApiUrl.ACCOUNTS_NOTE.format(id=id), params=params, response_obj=AccountNoteDetailsDTO())
        return response

    def address_uuid_details(self, id: str, uuid: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_ADDRESS_UUID.format(id=id, uuid=uuid), response_obj=AccountAddressUuidDetailsDTO())
        return response

    def note_uuid_details(self, id: str, uuid: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_NOTE_UUID.format(id=id, uuid=uuid), response_obj=AccountNoteUuidDetailsDTO())
        return response

    def note_uuid_files_details(self, id: str, uuid: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_NOTE_UUID_FILES.format(id=id, uuid=uuid), response_obj=AccountNoteUuidFileDetailsDTO())
        return response

    def note_uuid_files_uuid_details(self, id: str, uuid: str, file_uuid: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNTS_NOTE_UUID_FILES_UUID.format(id=id, uuid=uuid,file_uuid=file_uuid), response_obj=AccountNoteUuidFileUuidDetailsDTO())
        return response

    def details_information(self, id: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_INFORMATION.format(id=id), response_obj=AccountDetailsDTO())
        return response

    def cancel(self, id: str, request_data: AccountCancelDataDTO):
        cancel_request = AccountCancelDTO(account=request_data)
        response = self.post(url=AccountApiUrl.ACCOUNT_CANCEL.format(id=id), request_obj=cancel_request,
                             response_obj=AccountCancelResponseDTO())
        return response

    def reactivate(self, id: str, request_data: AccountReactivateDataDTO) -> AccountDetailsDTO:
        reactivate_request = AccountReactivateDTO(account=request_data)
        response = self.post(url=AccountApiUrl.ACCOUNT_REACTIVATE.format(id=id), request_obj=reactivate_request,
                             response_obj=AccountReactiveResponseDTO())
        return response

    def reactivate_v3(self, id: str, request_data: AccountReactivateDataDTO) -> AccountDetailsDTO:
        reactivate_request = AccountReactivateDTO(account=request_data)
        response = self.post(url=AccountApiUrl.ACCOUNT_REACTIVATE_V3.format(id=id), request_obj=reactivate_request,
                             response_obj=AccountReactiveResponseDTO())
        return response

    def delete(self, id: str):
        response = self.delete_request(url=AccountApiUrl.ACCOUNT_DELETE.format(id=id))
        return response

    def delete_image(self, id: str):
        response = self.delete_request(url=AccountApiUrl.ACCOUNTS_IMAGE.format(id=id))
        return response

    def delete_address_uuid(self, id: str, uuid: str):
        response = self.delete_request(url=AccountApiUrl.ACCOUNTS_ADDRESS_UUID.format(id=id, uuid=uuid))
        return response

    def delete_note_uuid(self, id: str, uuid: str):
        response = self.delete_request(url=AccountApiUrl.ACCOUNTS_NOTE_UUID.format(id=id, uuid=uuid))
        return response

    def delete_note_uuid_file_uuid(self, id: str, uuid: str, file_uuid: str):
        response = self.delete_request(url=AccountApiUrl.ACCOUNTS_NOTE_UUID_FILES_UUID.format(id=id, uuid=uuid,file_uuid=file_uuid))
        return response

    def contact_delete(self, id: str, contact_type: str):
        response = self.delete_request(
            url=AccountApiUrl.ACCOUNT_CONTACT_DELETE.format(id=id, contact_type=contact_type))
        return response

    def update_information(self, id: str, request_data: AccountUpdateInformationDTO) -> AccountDetailsDTO:
        response = self.patch(url=AccountApiUrl.ACCOUNT_UPDATE_INFORMATION.format(id=id), request_obj=request_data,
                              response_obj=AccountDetailsDTO())
        return response

    def update_address(self, id: str, uuid:str, request_data: AccountAddressRequestDTO) -> AccountAddressResponseDTO:
        response = self.patch(url=AccountApiUrl.ACCOUNTS_ADDRESS_UUID.format(id=id, uuid=uuid), request_obj=request_data,
                              response_obj=AccountAddressResponseDTO())
        return response

    def update_billing_preferences(self, id: str, request_data: AccountBillingPreferencesRequestDTO) -> AccountBillingPreferencesResponseDTO:
        response = self.put(url=AccountApiUrl.ACCOUNT_BILLING_PREFERENCE.format(id=id), request_obj=request_data,
                              response_obj=AccountBillingPreferencesResponseDTO())
        return response

    def get_contacts(self, id: str) -> AccountContactsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_CONTACTS.format(id=id), response_obj=AccountContactsDTO())
        return response

    def get_contact_type(self, id: str, contact_type: str) -> AccountContactsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_CONTACT_TYPE.format(id=id, contact_type=contact_type),
                            response_obj=AccountContactsDTO())
        return response

    def update_contact(self, id: str, contact_type: str,
                       request_data: AccountContactUpdateDTO) -> AccountContactsUpdateDTO:
        response = self.put(url=AccountApiUrl.ACCOUNT_CONTACT_UPDATE.format(id=id, contact_type=contact_type),
                            request_obj=request_data, response_obj=AccountContactsUpdateDTO)
        return response

    def update_contact_v3(self, id: str, contact_type: str,
                       request_data: AccountContactUpdateDTO) -> AccountContactsUpdateDTO:
        response = self.put(url=AccountApiUrl.ACCOUNT_CONTACT_UPDATE_V3.format(id=id, contact_type=contact_type),
                            request_obj=request_data, response_obj=AccountContactsUpdateDTO)
        return response

    def add_payment_method(self, account_id: str, request_data: PaymentMethodsAddDTO) -> PaymentMethodsDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNT_PAYMENT_METHODS.format(id=account_id), request_obj=request_data,
                             response_obj=PaymentMethodsDetailsDTO())
        return response

    def add_payment_card_method(self, account_id: str,
                                request_data: PaymentCardMethodsAddDTO) -> PaymentMethodsDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNT_PAYMENT_METHODS.format(id=account_id), request_obj=request_data,
                             response_obj=PaymentMethodsDetailsDTO())
        return response

    def add_payment_card_method_v3(self, account_id: str,
                                request_data: PaymentCardMethodsAddDTO) -> PaymentMethodsDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNT_PAYMENT_METHODS_V3.format(id=account_id), request_obj=request_data,
                             response_obj=PaymentMethodsDetailsDTO())
        return response

    def list_payment_method(self, account_id: str) -> PaymentMethodsListDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_PAYMENT_METHODS.format(id=account_id),
                            response_obj=PaymentMethodsListDTO())
        return response

    def delete_payment_method(self, account_id: str, reference: str):
        response = self.delete_request(
            url=AccountApiUrl.EACH_PAYMENT_METHODS.format(id=account_id, reference=reference))
        return response

    def delete_payment_method_v3(self, account_id: str, reference: str):
        response = self.delete_request(
            url=AccountApiUrl.EACH_PAYMENT_METHODS_V3.format(id=account_id, reference=reference))
        return response

    def payment_method_details(self, account_id: str, reference: str) -> PaymentMethodsDetailsDTO:
        response = self.get(url=AccountApiUrl.EACH_PAYMENT_METHODS.format(id=account_id, reference=reference),
                            response_obj=PaymentMethodsDetailsDTO())
        return response

    def billing_preference_details(self, account_id: str) -> AccountDetailsDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_BILLING_PREFERENCE.format(id=account_id),
                            response_obj=AccountDetailsDTO())
        return response

    def add_addresses(self, id: str, request_data: AccountAddressesAddDTO) -> AccountDetailsDTO:
        response = self.post(url=AccountApiUrl.ACCOUNT_ADDRESSES.format(id=id),
                             request_obj=request_data, response_obj=AccountDetailsDTO())
        return response

    def update_payment_method(self, id: str, reference: str,
                              request_data: PaymentMethodsAddDTO) -> PaymentMethodsDetailsDTO:
        response = self.patch(url=AccountApiUrl.ACCOUNT_PAYMENT_METHODS_UPDATE.format(id=id, reference=reference),
                              request_obj=request_data, response_obj=PaymentMethodsDetailsDTO())
        return response

    def modify_contact(self, id: str, contact_type: str,
                       request_data: AccountContactUpdateDTO) -> AccountContactsUpdateDTO:
        response = self.patch(url=AccountApiUrl.ACCOUNT_CONTACT_MODIFY.format(id=id, contact_type=contact_type),
                              request_obj=request_data, response_obj=AccountContactsUpdateDTO())
        return response

    def account_sent_email(self, file_url: str = None, account_id: str = "", email_data: dict = None):
        if not account_id:
            raise ValueError("Account ID is required")

        if not email_data or not isinstance(email_data, dict):
            raise ValueError("Invalid or missing 'email_data' parameter. Expected a dictionary.")

        cc_emails = ",".join(email_data.get("cc", [])) if email_data.get("cc") else ""
        bcc_emails = ",".join(email_data.get("bcc", [])) if email_data.get("bcc") else ""

        data = {
            "to": email_data.get("to", ""),
            "cc": cc_emails,
            "bcc": bcc_emails,
            "subject": email_data.get("subject", ""),
            "body": email_data.get("body", "")
        }

        files = None
        if file_url:
            mime_type, _ = mimetypes.guess_type(file_url)
            if not mime_type:
                raise ValueError(f"Unable to determine MIME type for file: {file_url}")

            file_name = os.path.basename(file_url)
            file = open(file_url, "rb")

            files = {
                "attachment": (file_name, file, mime_type)
            }

            try:
                response = self.post(
                    url=AccountApiUrl.ACCOUNT_SENT_EMAIL.format(id=account_id),
                    data=data,
                    file=files,
                    response_obj=EmailSentResponseDTO()
                )
                return response
            finally:
                if file_url:
                    file.close()

        else:
            response = self.post(
                url=AccountApiUrl.ACCOUNT_SENT_EMAIL.format(id=account_id),
                data=data,
                response_obj=EmailSentResponseDTO()
            )

        return response

    def account_get_email(self, id: str) -> EmailGetListResponseDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_SENT_EMAIL.format(id=id), response_obj=EmailGetListResponseDTO())
        return response

    def account_get_email_uuid(self, id: str, uuid: str) -> EmailGetDetailsResponseDTO:
        response = self.get(url=AccountApiUrl.ACCOUNT_SENT_EMAIL_UUID.format(id=id,uuid=uuid), response_obj=EmailGetDetailsResponseDTO())
        return response



from order_usage_db.connect_with_db import DatabaseConnection
from order_usage_db.order_service import OrderService


class OrderManager:
    def __init__(self, db_name, user, password, host):
        self.db_connection = DatabaseConnection(db_name, user, password, host)
        self.order_service = None

    def connect_to_db(self):
        self.db_connection.connect()
        self.order_service = OrderService(self.db_connection)

    def disconnect_from_db(self):
        self.db_connection.close()

    def process_order(self, account_id: str, order_id: str, item_id: str, item_name: str, charge_item_uuid: str):
        new_order = self.order_service.create_association_data(
            account_id=account_id,
            order_id=order_id,
            item_id=item_id,
            item_name=item_name,
            charge_item_uuid=charge_item_uuid
        )
        print(f"Created Order ID: {new_order.id}")

    def fetch_order(self, order_id: str):
        order = self.order_service.get_order_by_id(order_id)
        if order:
            print(f"Order Found: {order.item_name}")
        else:
            print("Order not found.")



# express_dto.py

from dataclasses import dataclass
from typing import List
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class PricingRuleDTO(ABBaseDTO):
    price: str = None


@dataclass(kw_only=True)
class ItemPriceSnapshotDTO(ABBaseDTO):
    pricingRule: PricingRuleDTO = None


@dataclass(kw_only=True)
class ItemPriceTaxDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: float = None

@dataclass(kw_only=True)
class PaymentAppliedDTO(ABBaseDTO):
    processor: str = None
    amount: str = None

@dataclass(kw_only=True)
class PaymentDTO(ABBaseDTO):
    paymentApplied: List[PaymentAppliedDTO] = None

@dataclass(kw_only=True)
class InvoiceDTO(ABBaseDTO):
    payment: PaymentDTO = None

@dataclass(kw_only=True)
class ContractPropertiesDTO(ABBaseDTO):
    requireCustomerAcceptance: str = None
    requiresPaymentMethod: str = None
    initialContractTerm: str = None
    renewAutomatically: str = None
    autoRenewalTerm: str = None
    allowEarlyTermination: str = None
    applyEarlyTerminationCharge: str = None
    allowPostponement: str = None
    maximumDurationPerPostponement: str = None
    maximumPostponementCount: str = None
    allowTrial: str = None
    startContractAfterTrialEnds: str = None
    trialPeriod: str = None
    allowDowngrade: str = None
    periodBeforeDowngrade: str = None
    allowDowngradeCharge: str = None
    downgradeChargeType: str = None
    downgradeChargeFixed: str = None
    allowUpgrade: str = None

@dataclass(kw_only=True)
class OrderLineDTO(ABBaseDTO):
    itemId: str = None
    itemOrderQuantity: int = None
    itemPriceSnapshot: ItemPriceSnapshotDTO = None
    itemPriceTax: ItemPriceTaxDTO = None
    packageName: str = None

@dataclass(kw_only=True)
class OrderDTO(ABBaseDTO):
    lines: List[OrderLineDTO] = None
    invoice: InvoiceDTO = None
    allowContract: str = None
    contractProperties: ContractPropertiesDTO = None
    properties: dict = None

@dataclass(kw_only=True)
class AccountDTO(ABBaseDTO):
    id: str = None
    order: OrderDTO = None

@dataclass(kw_only=True)
class ExpressDTO(ABBaseDTO):
    account: AccountDTO = None



from exsited.exsited.common.common_enum import SortDirection
from exsited.common.sdk_util import SDKUtil
from exsited.exsited.purchase_payments.dto.purchase_payments_dto import PurchasePaymentsListDTO, \
    PurchasePaymentsDetailsDTO, PurchasePaymentUpdateRequestDTO, PurchasePaymentsUpdateResponseDTO, PurchasePaymentRequestDTO, PurchasePaymentResponseDTO
from exsited.exsited.purchase_payments.purchase_payments_api_url import PurchasePaymentsApiUrl

from exsited.http.ab_rest_processor import ABRestProcessor


class PurchasePayments(ABRestProcessor):

    def list(self, limit: int = None, offset: int = None, direction: SortDirection = None,
             order_by: str = None) -> PurchasePaymentsListDTO:
        params = SDKUtil.init_pagination_params(limit=limit, offset=offset, direction=direction, order_by=order_by)
        response = self.get(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS, params=params, response_obj=PurchasePaymentsListDTO())
        return response

    def details(self, id: str) -> PurchasePaymentsDetailsDTO:
        response = self.get(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS_DETAILS.format(id=id),
                            response_obj=PurchasePaymentsDetailsDTO())
        return response

    def update(self, purchase_payment_id: str, request_data: PurchasePaymentUpdateRequestDTO) -> PurchasePaymentsUpdateResponseDTO:
        response = self.patch(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS_DETAILS.format(id=purchase_payment_id),
                              request_obj=request_data, response_obj=PurchasePaymentsUpdateResponseDTO())
        return response

    def delete(self, purchase_payment_id: str) -> dict:
        response = self.delete_request(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS_DETAILS.format(id=purchase_payment_id), response_obj={})
        return response

    def create(self, request_data: PurchasePaymentRequestDTO) -> PurchasePaymentResponseDTO:
        response = self.post(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS, request_obj=request_data, response_obj=PurchasePaymentResponseDTO())
        return response

    def create_from_purchase_invoice(self, purchase_invoice_id: str, request_data: PurchasePaymentRequestDTO) -> PurchasePaymentResponseDTO:
        response = self.post(url=PurchasePaymentsApiUrl.PURCHASE_PAYMENTS_FROM_PURCHASE_INVOICE.format(purchase_invoice_id=purchase_invoice_id), request_obj=request_data, response_obj=PurchasePaymentResponseDTO())
        return response



from dataclasses import dataclass
from exsited.exsited.common.dto.common_dto import TaxDTO
from exsited.sdlize.ab_base_dto import ABBaseDTO


@dataclass(kw_only=True)
class OrderItemAccountingCodeDTO(ABBaseDTO):
    salesRevenue: str = None


@dataclass(kw_only=True)
class OrderItemPricingRuleDTO(ABBaseDTO):
    price: str
    uuid: str = None
    price: str = None
    version: str = None
    priceType: str = None
    uom: str = None
    pricePeriod: str = None
    pricingSchedule: str = None
    pricingLevel: str = None
    pricingMethod: str = None
    warehouse: str = None


@dataclass(kw_only=True)
class OrderItemPropertiesDTO(ABBaseDTO):
    billingMode: str = None
    chargingPeriod: str = None
    chargingStartDate: str = None
    chargingAndBillingAlignment: bool = None
    proRataPartialChargingPeriod: bool = None
    proRataPartialPricingPeriod: bool = None


@dataclass(kw_only=True)
class OrderItemSaleTaxConfigurationDTO(ABBaseDTO):
    salePriceIsBasedOn: str = None
    taxCode: TaxDTO = None


@dataclass(kw_only=True)
class OrderItemPriceSnapshotDTO(ABBaseDTO):
    pricingRule: OrderItemPricingRuleDTO = None

    def add_rule(self, price: str):
        self.pricingRule = OrderItemPricingRuleDTO(price=price)
        return self


@dataclass(kw_only=True)
class POInformationDTO(ABBaseDTO):
    id: str = None
    name: str = None
    accountId: str = None
    currency: str = None
    itemQuantity: str = None
    itemPriceSnapshot: OrderItemPriceSnapshotDTO = None


@dataclass(kw_only=True)
class OrderPurchaseDTO(ABBaseDTO):
    createPo: str = None
    poInformation: POInformationDTO = None

@dataclass(kw_only=True)
class PreOrderStockDetailsDTO(ABBaseDTO):
    pendingReserve: str = None
    reserved: str = None
    sold: str = None


@dataclass(kw_only=True)
class DiscountProfileDTO(ABBaseDTO):
    name: str = None
    uuid: str = None
    type: str = None
    amount: str = None
    redemptionCode: str = None


@dataclass(kw_only=True)
class SalesRevenueDetailsDTO(ABBaseDTO):
    salesRevenue: str = None

@dataclass(kw_only=True)
class itemCustomAttributesDataDTO(ABBaseDTO):
    name: str = None
    value: str = None

@dataclass(kw_only=True)
class AccountingCodeDTO(ABBaseDTO):
    salesRevenue: str = None

@dataclass(kw_only=True)
class OrderLineDTO(ABBaseDTO):
    itemId: str = None
    itemOrderQuantity: str = None
    itemUuid: str = None
    itemName: str = None
    shippingCost: str = None
    itemInvoiceNote: str = None
    itemDescription: str = None
    itemType: str = None
    itemChargeType: str = None
    chargeItemUuid: str = None
    version: str = None
    itemPriceTax: TaxDTO = None
    isTaxExemptWhenSold: str = None

    itemPriceSnapshot: OrderItemPriceSnapshotDTO = None
    itemSaleTaxConfiguration: OrderItemSaleTaxConfigurationDTO = None
    purchaseOrder: OrderPurchaseDTO = None
    packageName: str = None
    itemSerialOrBatchNumber: str = None
    itemCustomAttributes: list[itemCustomAttributesDataDTO] = None
    op: str = None
    uuid: str = None
    itemProperties: OrderItemPropertiesDTO = None

    subtotal: str = None
    total: str = None
    tax: TaxDTO = None

    itemUom: str = None
    itemWarehouse: str = None
    pricingSnapshotUuid: str = None
    chargingStartDate: str = None
    alternateChargingStartDate: str = None
    chargingEndDate: str = None
    alternateChargingEndDate: str = None
    uuid: str = None

    itemPrice: str = None
    itemDiscountAmount: str = None
    taxAmount: str = None
    operation: str = None


    discount: str = None
    uom: str = None
    warehouse: str = None
    quantity: str = None
    try:
        accountingCode: AccountingCodeDTO = None
    except:
        accountingCode: str = None

    itemAccountingCode: OrderItemAccountingCodeDTO = None
    preOrderStockDetails: PreOrderStockDetailsDTO = None
    expectedDeliveryDate: str = None



@dataclass(kw_only=True)
class OrderPropertiesDTO(ABBaseDTO):
    communicationProfile: str = None
    invoiceMode: str = None
    invoiceTerm: str = None
    billingPeriod: str = None
    paymentProcessor: str = None
    paymentMode: str = None
    paymentTerm: str = None
    paymentTermAlignment: str = None
    fulfillmentMode: str = None
    fulfillmentTerm: str = None
    consolidateInvoice: str = None
    consolidateKey: str = None


@dataclass(kw_only=True)
class ContractPropertiesDTO(ABBaseDTO):
    requireCustomerAcceptance: str = None
    requiresPaymentMethod: str = None
    initialContractTerm: str = None
    renewAutomatically: str = None
    autoRenewalEndsOn: str = None
    autoRenewalRequireCustomerAcceptance: str = None
    autoRenewalTerm: str = None
    allowEarlyTermination: str = None
    earlyTerminationMinimumPeriod: str = None
    applyEarlyTerminationCharge: str = None
    earlyTerminationChargeType: str = None
    terminationPercentageCharge: str = None
    terminationAccountingCode: str = None
    allowPostponement: str = None
    maximumDurationPerPostponement: str = None
    maximumPostponementCount: str = None
    allowTrial: str = None
    startContractAfterTrialEnds: str = None
    trialPeriod: str = None
    trialEndDate: str = None
    trialRequiresPaymentMethod: str = None
    allowDowngrade: str = None
    periodBeforeDowngrade: str = None
    allowDowngradeCharge: str = None
    downgradeChargeType: str = None
    downgradeChargeFixed: str = None
    downgradeChargePercentage: str = None
    downgradeBillingPeriodCount: str = None
    allowUpgrade: str = None
    terminationFixedCharge: str = None
    terminationNoticePeriod: str = None





@dataclass(kw_only=True)
class OrderItemPriceTaxDTO(ABBaseDTO):
    uuid: str = None
    code: str = None
    rate: str = None

@dataclass(kw_only=True)
class UpgradeDowngradePreviewDTO(ABBaseDTO):
    subTotal: str = None
    taxTotal: str = None
    discountTotal: str = None
    shippingTotal: str = None
    total: str = None
    currency: str = None


@dataclass(kw_only=True)
class ContractAdjustmentPreviewDTO(ABBaseDTO):
    subTotal: str = None
    taxTotal: str = None
    discountTotal: str = None
    shippingTotal: str = None
    total: str = None
    currency: str = None
    oldTotal: str = None
    totalChanged: str = None
    totalDue: str = None


@dataclass(kw_only=True)
class KpisDTO(ABBaseDTO):
    startDate: str = None
    estimatedTotal: float = None
    totalRevenue: float = None
    monthlyRecurringRevenue: float = None
    totalCollected: float = None
    totalOutstanding: float = None
    totalDue: float = None
    lastInvoiceIssueDate: str = None
    lastInvoiceTotal: float = None
    totalInvoice: float = None
    nextInvoiceIssueDate: str = None
    lastReactivatedOn: str = None
    lastCancelledOn: str = None
    lastChangedOn: str = None
    lastDeletedOn: str = None
